from . import logger
from .utils import unescape
import os
import subprocess
import requests

class Downloader():
    def __init__(self, course_data, root_dir):
        self.course_data = course_data
        self.root_dir = root_dir

    def __create_directory(self, dir_name):
        try:
            os.mkdir(dir_name)
        except FileExistsError:
            pass


    def download(self):        
        self.__create_directory(self.root_dir)

        current_chapter = 0
        for data in self.course_data:
            if current_chapter < data['chapter_number']:
                current_chapter += 1
                dir_name = '{0:02d} - {1}'.format(current_chapter, data['chapter'])
                directory = os.path.join(self.root_dir, unescape(dir_name))
                self.__create_directory(directory)
            
            if data['data_type'] == 'Video':
                link = data['data_urls']
                filename = '{0:02d} - {1}.mp4'.format(data['lecture_number'], data['lecture'])
                self.__get_data(directory, link, unescape(filename))
            
            if data['attached_info']['attached_list']:
                """ Download attachments """                
                attachments = data['attached_info']['attached_list']
                for attachment in attachments:
                    link = attachment['link']
                    filename = attachment['filename']
                    self.__get_data(directory, link, unescape(filename))                

    def __get_data(self, directory, link, filename):        
        os.chdir(directory)
        logger.info('Downloading lecture: %s', filename)
        self.__curl_dl(link, filename)

    def __curl_dl(self, link, filename):
        command = ['curl', '-C', '-', link, '-o', filename, '--progress-bar']

        cert_path = requests.certs.where()
        if cert_path:
            command.extend(['--cacert', cert_path])
        else:
            command.extend(['--insecure'])
        subprocess.call(command)
    


import platform
import sys

def sys_info():
    result = {
        'platform': '{} [{}]'.format(platform.platform(), platform.version()),
        'python': '{} {}'.format(
            platform.python_implementation(),
            sys.version.replace('\n', '')
        ),
        'os': 'Unknown'
    }

    linux_ver = platform.linux_distribution()
    mac_ver = platform.mac_ver()
    win_ver = platform.win32_ver()

    if linux_ver[0]:
        result['os'] = 'Linux - {}'.format(' '.join(linux_ver))
    elif mac_ver[0]:
        result['os'] = 'OS X - {}'.format(' '.join(mac_ver[::2]))
    elif win_ver[0]:
        result['os'] = 'Windows - {}'.format(' '.join(win_ver[:2]))

    return result

def unescape(string):
    """Replace some characters to nothing"""
    string = string.replace('/', '')
    string = string.replace('->', '')
    string = string.replace('(', '')
    string = string.replace(')', '')
    return string



from . import __title__
from .exceptions import UdemyException
from . import COURSE_TITLE_URL, COURSE_INFO_URL, GET_LECTURE_URL, ATTACHMENT_URL, GET_LECTURE_URL
from . import logger
import re

class Course(object):
    def __init__(self, course_link, session):
        self.course_link = course_link
        self.session = session
        self.course_id = self.__get_course_id(self.course_link)

    def __get_course_id(self, course_url):
        """Extract course_id from the course page"""
        response = self.session.get(course_url)
        response_text = response.text

        if 'data-purpose="take-this-course-button"' in response_text:            
            raise UdemyException('You\'re not enrolled in this course')
        
        logger.debug('Searching course id...')

        matches = re.search(r'data-course-id="(\d+)"', response_text, re.IGNORECASE)
        if matches:
            course_id = matches.groups()[0]
        else:
            matches = re.search(r'property="og:image"\s+content="([^"]+)"', response_text, re.IGNORECASE)
            course_id = matches.groups()[0].rsplit('/', 1)[-1].split('_', 1)[0] if matches else None

        if not course_id:
            raise UdemyException('Course id not found!')
        else:
            logger.debug('Found course id: %s', course_id)

        return course_id

    def get_course_data(self, quality):        
        course_url = COURSE_INFO_URL.format(course_id=self.course_id)
        json_source = self.session.get(course_url).json()

        chapter = None
        course_data = []
        supported_asset_type = ['Video', 'VideoMashup']
        supported_supplementary_assets = ['File']

        lecture_number = 1
        chapter_number = 0
        item_count = 0

        for item in json_source['results']:
            item_count += 1
            lecture = item['title']
            lecture_id = item['id']           

            if item['_class'] == 'chapter':                
                chapter = item['title']
                chapter_number += 1

            elif item['_class'] == 'lecture' and item['asset']['asset_type'] in supported_asset_type:                           
                try:
                    data_urls, data_type = self.__extract_lecture_url(self.course_id, lecture_id, quality)                   

                    if data_urls is None:
                        lecture_number += 1
                        continue

                    attached_list = []
                    if item.get('supplementary_assets'):
                        for asset in item['supplementary_assets']:
                            if asset['asset_type'] in supported_supplementary_assets:
                                attached_list.append({
                                    'filename': asset['filename'],
                                    'id': asset['id'],
                                    'link': self.__parse_file(self.course_id, lecture_id, asset['id'])
                                })

                    attached_info = {
                        'course_id': self.course_id,
                        'lecture_id': lecture_id,
                        'attached_list': attached_list
                    }

                    course_data.append({
                        'chapter': chapter,
                        'lecture': lecture,
                        'data_urls': data_urls,
                        'data_type': data_type,
                        'attached_info': attached_info,
                        'lecture_number': int(lecture_number),
                        'chapter_number': int(chapter_number)
                    })
                except Exception as e:
                    logger.debug('Cannot download lecture "%s": "%s"', lecture, e)

                lecture_number += 1        
        return course_data

    def __extract_lecture_url(self, course_id, lecture_id, quality):
        """Extracting Video URLs from json_source for type File."""
        get_url = GET_LECTURE_URL.format(course_id=course_id, lecture_id=lecture_id)
        json_source = self.session.get(get_url).json()
        
        dict_videos = {}

        if json_source['asset']['download_urls']:
            logger.debug('Found videos marked as downloadable: %s', lecture_id)
            for video in json_source['asset']['download_urls']['Video']:
                dict_videos[video['label']] = video['file']
            return (dict_videos[quality], 'Video')

        elif json_source['asset']['stream_urls']:
            logger.debug('Falling back to stream urls: %s', lecture_id)
            for video in json_source['asset']['stream_urls']['Video']:
                dict_videos[video['label']] = video['file']
            return (dict_videos[quality], 'Video')

        else:            
            logger.debug("Couldn't extract lecture url: %s, the lecture might be set as not downloadable", lecture_id)
            return (None, None)

    def __parse_file(self, course_id, lecture_id, attachment_id):
        """Extracting URL from json_source for type File."""
        get_url = ATTACHMENT_URL.format(course_id=course_id, lecture_id=lecture_id, attachment_id=attachment_id)
        json_source = self.session.get(get_url).json()
        for f in json_source['download_urls']['File']:
            if f['label'] == 'download':
                return f['file']
            else:
                logger.debug("Skipped. Couldn't fetch File!")
                return None
    